##::xo::db::require package xowf
::xo::library require -package xowf xowf-procs

##simple classes for grading procs

Class create GenericGradingType

GenericGradingType instproc validateInput {value} {
    ns_log notice "validationmethod: none found, revert to superclass"
    return 1
}

###
### note
###
Class create GradingType=note -superclass GenericGradingType -parameter  {grade_scheme}

GradingType=note instproc PrettyName {args} {
    return "[_ tlf-assignments.type_grade]"
}

GradingType=note proc asOption {} {
    return [subst {{[_ tlf-assignments.type_grade]} {note}}]
}

GradingType=note instproc asHTMLInput {submission} {
    set options ""
    if {[$submission set state] eq "submitted" || [$submission set state] eq "grade_state"} {
        for {set i 1} {$i <=5} {incr i} {
            if {[$submission get_property -name grade] ne $i} {
                append options "<option>$i</option>"
            } else {
                append options "<option selected>$i</option>"
            }
        }
        set ratinghtml "
        <select name='grading' id='[$submission set item_id]'>
        <option>--</option>
        $options
        </select>"
    } else {
        set ratinghtml [$submission get_property -name grade]
    }
    return $ratinghtml
}

GradingType=note instproc formConst {args} {
    set fc "{grade:select,options={1 1} {2 2} {3 3} {4 4} {5 5},label=#tlf-assignments.grading#,required}"
    return $fc
}

GradingType=note instproc validateInput {args} {
    set value [lindex $args 0 0]
    if {[string is integer $value] && $value <=5 && $value > 0} {
        return 1
    } else {
        util_user_message -message "[_ tlf-assignments.grade_error]"
        return 0
    }
}

GradingType=note proc allowQuickGrade {} {
    return 1
}

####
###points
####
Class create GradingType=points -superclass GenericGradingType -parameter {maxpoints}

GradingType=points proc asOption {} {
    return {{#tlf-assignments.points# max.} {points}}
}

GradingType=points instproc PrettyName {args} {
    return "[_ tlf-assignments.points]"
    #"von max [$args get_property -name points]"
}

GradingType=points instproc asHTMLInput {submission} {
    #todo display max points
    set ast ::[$submission get_property -name assignment_id]
    set maxpoints  [$ast get_property -name points]
    set grade [$submission get_property -name grade]
    if {[$submission set state] eq "submitted" || [$submission set state] eq "grade_state"} {
        set ratinghtml "
            <input name='grading' id='[$submission set item_id]' 
            size='3' value='$grade'>[:PrettyName $ast] [_ tlf-assignments.of] $maxpoints
        "
    } else {
        set ratinghtml "$grade [:PrettyName $ast] [_ tlf-assignments.of] $maxpoints"
    }
    return $ratinghtml
}

GradingType=points instproc formConst {args} {
    set maxpoints [$args get_property -name points -default 10]
    set fc "{grade:points,size=4,label=#tlf-assignments.points# #tlf-assignments.of# $maxpoints,required,validator=grading_validator}"
    return $fc
}

GradingType=points instproc validateInput {args} {
    set value [lindex $args 0 0]
    set points [lindex $args 0 1]
    if {[lc_get "decimal_point"] eq ","} {regsub "," $value "." value}
    if {(![string is double $value]) || $value < 0} {
        util_user_message -message "[_ tlf-assignments.points_lt_0]"
        return 0
        }
    if {$value > $points && $points ne ""} {
        util_user_message -message "[_ tlf-assignments.maxpoints] $points"
        return 0
    }
    if {$value != [format "%.2f" $value]} {
        util_user_message -message "[_ tlf-assignments.commas]"
        return 0
    }
    return 1
}

GradingType=points proc allowQuickGrade {} {
     return 1
}

###
### percent
###
Class create GradingType=percent -superclass GenericGradingType -parameter {}

GradingType=percent proc asOption {} {
    return {{#tlf-assignments.percent#} {percent}}
}

GradingType=percent instproc PrettyName {args} {
    return "[_ tlf-assignments.percent]"
}

GradingType=percent instproc asHTMLInput {submission} {
    set grade [$submission get_property -name grade]
    if {[$submission set state] eq "submitted" || [$submission set state] eq "grade_state" } {
        set ratinghtml "
            <input accesskey='b' name='grading' id='[$submission set item_id]' 
            size='3' value='$grade'>%
        "
    } else {
        set ratinghtml "${grade} %"
    }
    return $ratinghtml
}

GradingType=percent instproc formConst {args} {
    set fc "{grade:points,size=4,validator=grading_validator,label=#tlf-assignments.percent#,required}"
    return $fc
}

GradingType=percent instproc validateInput {args} {
    set value [lindex $args 0 0]
    if {[lc_get "decimal_point"] eq ","} {regsub "," $value "." value}
    if {![string is double $value] || $value > 100 || $value < 0} {
        util_user_message -message "[:PrettyName] [_ tlf-assignments.percent_gt_100]"
        return 0
    }
    if {$value != [format "%.2f" $value]} {
        util_user_message -message "#tlf-assignments.commas#"
        return 0
    }
    return 1
}

GradingType=percent proc allowQuickGrade {} {
    return 1
}

###
### passed
###
Class create GradingType=passed -superclass GenericGradingType -parameter  {}

GradingType=passed proc asOption {} {
    return {{#tlf-assignments.type_passed#} {passed}}
}

GradingType=passed instproc PrettyName {args} {
    return "[_ tlf-assignments.type_passed]"
}

GradingType=passed instproc asHTMLInput {submission} {
    set grade [$submission get_property -name grade]
    if {[$submission set state] eq "submitted" || [$submission set state] eq "grade_state"} {
        if {$grade eq "passed"} {
            set options "<option selected>bestanden[_ tlf-assignments.passed]</option>
                                <option>[_ tlf-assignments.not_passed]</option>"
        } elseif {$grade eq "not passed"} {
            set options "<option>[_ tlf-assignments.passed]</option>
            <option selected>[_ tlf-assignments.not_passed]</option>"
        } else {
          set options "
        <option>[_ tlf-assignments.passed]</option>
        <option>[_ tlf-assignments.not_passed]</option>"
        }
        set ratinghtml "
        <select name='grading' id='[$submission set item_id]'>
        $options
        </select>
        "
        return $ratinghtml
    } else {
        # return [$submission get_property -name grade]
        if {$grade in [list [lang::message::lookup de tlf-assignments.passed] \
            [lang::message::lookup en tlf-assignments.passed] "passed"]} {
            return [_ tlf-assignments.passed]
        } else {
            return [_ tlf-assignments.not_passed]
        }
    }
}

GradingType=passed instproc formConst {args} {
    set fc [subst -nocommands "{grade:select,options={#tlf-assignments.passed# passed} 
    {{#tlf-assignments.not_passed#} {not passed}},label=#tlf-assignments.grading#,required}"]
    return $fc
}

GradingType=passed instproc validateInput {value} {
    #todo: check for invalid values
    return 1
}

GradingType=passed proc allowQuickGrade {} {
    return 1
}



###
### comment
###
Class create GradingType=comment -superclass GenericGradingType -parameter {}

GradingType=comment proc asOption {} {
    return {{#tlf-assignments.type_comment#} {comment}}
}

GradingType=comment instproc PrettyName {args} {
    return "[_ tlf-assignments.type_comment]"
}

GradingType=comment instproc formConst {args} {
    set fc "grade:hidden"
    return $fc
}

GradingType=comment instproc asHTMLInput {submission} {
    #stupid, change this
    return [:PrettyName]
}

GradingType=comment proc allowQuickGrade {} {
    return 0
}

###
### no_grading
###
Class create GradingType=no_grading -superclass GenericGradingType -parameter {}

GradingType=no_grading proc asOption {} {
     return {{#tlf-assignments.no_grading#} {no_grading}}
}

GradingType=no_grading instproc PrettyName {args} {
    return "[_ tlf-assignments.no_grading]"
}

GradingType=no_grading instproc formConst {args} {
    set fc "grade:hidden"
    return $fc
}

GradingType=no_grading instproc asHTMLInput {submission} {
    return "[_ tlf-assignments.no_grading_allowed]"
}

GradingType=no_grading proc allowQuickGrade {} {
    return 0
}

