##::xo::db::require package xowiki

namespace eval ::xowiki {

    File ad_instproc -private privilege=authorized_stud_4file {
        {-login true}
        user_id
        package_id
        method
    } {
        Define an object specific privilege to be used in the policies.
        Only students entitled to see this file can see it
    } {
        return [${:parent_id} privilege=authorized_stud $user_id $package_id $method]
    }

    FormPage ad_instproc -private privilege=authorized_stud {
        {-login true}
        user_id
        package_id
        method
    } {
        Define an object specific privilege to be used in the policies.
        Only students entitled to see this page can see it
    } {
        if {[string match "create-assignment\[0-9\]*" ${:name}]} {
            return 1
        }
        set ast [:get_property -name assignment_id]
        if {$ast eq ""} {
            return 0
        }
        if {$user_id == 0} {
            set user_id [auth::require_login]
        }

        set item_id ${:item_id}

        return [::xo::dc 0or1row -prepare integer,integer,integer,integer is_authorized {
            select 1 from dual where exists
            (select 1
              from xowiki_form_instance_item_index i
            where
                  -- user_id is either the creator or group for this page
                  (i.item_id = :item_id and
                   (:user_id in (cast(i.hkey->'creator_id' as integer),
                                 cast(i.hkey->'group_id' as integer),
                                 (select creation_user from acs_objects where object_id = i.item_id))
                    )
                   )
             or
                  -- user_id is member of one of the groups of the assignment
                  (i.item_id = :ast and
                   exists (select 1 from group_member_map
                            where group_id = any(cast(regexp_split_to_array(i.hkey->'groups', E'\\s+') as integer[]))
                              and member_id = :user_id))
             )
        }]
    }

    Policy __tlf_assignments -contains {
        ##extends the survey policy with special methods for podcasts
        Class create Package -array set require_permission {
            reindex             {{id admin}}
            change-page-order   {{id admin}}
            manage-categories   {{id admin}}
            edit-category-tree  {{id admin}}
            show-object   admin
            serialize         admin
            delete              admin
            edit-new            admin
            create-assign    admin
        }

        Class create File -array set require_permission {
            download 	authorized_stud_4file
            view        authorized_stud_4file
            show-object swa
            delete  creator
        }

        Class create Object -array set require_permission {
            view login
            edit admin
        }

        Class create Page -array set require_permission {
            view {
                {{regexp {name {(groupmgr|importer)$}}} package_id admin}
                login
            }
            revisions          swa
            diff               swa
            edit               admin
            make-live-revision swa
            save-attributes    swa
            delete-revision    admin
            delete             swa
            create-new        swa
            create-or-use     admin
            create-comment login
            edit-new  admin
        }

        Class create Form -array set require_permission {
            serialize   admin
            list    admin
        }

        Class create FormPage -array set require_permission {
            create-new        admin
            delete            admin
            view      authorized_stud
            create-or-use     admin
            list              admin
            edit     login
            serialize               swa
            show-object   swa
            render_exercise login
            submit  login
            foreign_submit  admin
            bulk_grade  admin
            bulk_download   admin
            write_nf    admin
            dosubmit    login
            eph_register    admin
            list_answers    admin
            csv_export  admin
            csv_export_ot  admin
            view_composite  login
            unassign_group admin
            set_visibility  admin
            release_grade admin
        }
    }
}

::xo::library source_dependent
