ad_library {
    tlf-assignments core

    @creation-date 2011-02-22
    @author Markus Moser
}

##::xo::db::require package xowf
::xo::library require -package xowf xowf-procs
::xo::library require -package tlf-assignments tlf-assignments-wf-procs
namespace eval ::tlf_cassignments {

    ::xo::PackageMgr create ::tlf_cassignments::Package \
        -package_key "tlf-assignments" \
        -pretty_name "#tlf-assignments.mod_title#" \
        -superclass ::xowf::Package -table_name "tlf_assignments_core"

    ad_proc -private before-install {} {
        #check if hstore is available
        if {![::xo::dc has_hstore]} {
            #get rid of registered package, abort installation
            ::xo::dc get_value unregister {
                select apm_package_type__drop_type('tlf-assignments', 't');
            }
            error "hstore not found. This package requires hstore for postgres.\
                Please consult the README for installation instructions."
        }
    }
    
    ad_proc -private before-uninstall {} {
        ast_installer uninstall
    }

    ad_proc -private after-install {} {
        ast_installer install
    }


    ad_proc -private before-uninstantiate {
         {-package_id:required}
    } {
       Callback to be called whenever a package instance is deleted.
    } {
        set applet_key [ast_installer applet_key]
        ::xo::dc dml delete_dotlrn_community_applet {
            delete from dotlrn_community_applets
            where package_id = :package_id
            and applet_id = (select applet_id
                             from dotlrn_applets
                             where applet_key = :applet_key)}
        ::xowiki::before-uninstantiate -package_id $package_id
    }

    ad_proc -private mount_callback {
        -package_id:required
        -node_id:required
    } {
        #create objects in the newly mounted package?
        return 1
    }

    ad_proc -private upgrade_callback {
        {-from_version_name:required}
        {-to_version_name:required}
    } {
        Upgrade logic
    } {
        apm_upgrade_logic \
            -from_version_name $from_version_name \
            -to_version_name $to_version_name \
            -spec {
                0.131d 0.133d {
                    ::xo::dc transaction {
                        #placeholder
                    }
                }
            }
    }

    ad_proc -private user_role {
        {-package_id ""}
        {-community_id ""}
    } {
        Checks if the current user has admin privileges for the
        package

        @author Monika Andergassen
    } {
        # get default values
        if { $package_id=="" } { set package_id [ad_conn package_id] }
        if { $community_id=="" } { set community_id [dotlrn_community::get_community_id] }
        # check if user has admin privileges
        if { [permission::permission_p -object_id $package_id -privilege "admin"] } {
            return "admin"
        }
        # check if user has student privileges
        if { "student"==[dotlrn_community::get_user_role $community_id] } {
            return "student"
        } else {
            return "member"
        }
    }

    namespace eval ::xowiki {
        ::xowiki::MenuBar instproc config=assignments {
            {-bind_vars {}}
            -current_page:required
            -package_id:required
            -folder_link:required
            -return_url
        } {
            ::xowiki::includelet::folder_config::assignments -mb [:self] -page $current_page
            return {}

        }
    }


    namespace eval ::xowiki::includelet::folder_config {}

    ad_proc -private ::xowiki::includelet::folder_config::assignments {
        {-mb:required}
        {-page:required}
    } {
        Survey folder configuration
    } {
        set package_id [$page package_id]
        set community_id [dotlrn_community::get_community_id]
        set key [ast_installer applet_key]
        set orig_url ""
        set package_url [$package_id pretty_link -parent_id [$package_id folder_id] ""]
        if {![permission::permission_p -object_id $package_id \
                  -privilege "admin"]} {return}
        $mb add_menu -name Aufgaben -label #tlf-assignments.asts#
        $mb add_menu_item -name Aufgaben.Startseite \
            -item [list label "[_ tlf-assignments.start_page]" url $package_url]
        $mb add_menu -name Neu -label #tlf-assignments.new#
        $mb add_menu_item -name Neu.Aufgabe  \
            -item [list label "[_ tlf-assignments.create_new]" \
                       url "${package_url}create-assignment.wf?m=create-new"]
        $mb add_menu -name Gruppen -label "[_ tlf-assignments.groupmgr]"
        $mb add_menu_item -name Gruppen.Gruppen\
            -item [list label "[_ tlf-assignments.groupmgr_action]" \
                       url "${package_url}groupmgr"]
        $mb add_menu_item -name Gruppen.Export \
            -item [list label "[_ tlf-assignments.Group_export]" \
                       url "${package_url}groupmgr?a=export"]
        set context_item 0
        if {$context_item ne 0} {
            mb add_menu -name Page -label "[_ tlf-assignments.This_assignment]"
            mb add_menu_item -name Page.Bearbeiten\
                -item [list label "[_ tlf-assignments.Edit_this_assignment]" \
                           url "[$context_item pretty_link]?m=edit"]
            if {[clock scan [$context_item get_property -name time_from]] < [clock seconds] } {
                $mb add_menu_item -name Page.CSV \
                    -item [list label "[_ tlf-assignments.csv_export]" \
                               url "[$context_item pretty_link]?m=csv_export"]
                $mb add_menu_item -name Page.CSVall \
                    -item [list label "[_ tlf-assignments.csv_export_all]" \
                               url "[$context_item pretty_link]?m=csv_export&all=1"]
            }
        }
        return $mb
    }

    Package ad_instproc -private get_parameter {
        {-check_query_parameter true}
        {-type ""}
        attribute
        {default ""}
    } {
        Hardcoded parameters
    } {
        switch $attribute {
            security_policy {
                return "::xowiki::__tlf_assignments"
            }
            index_page {
                return "index"
            }
            top_includelet {
                return "none"
            }
            use_hstore {
                return 1
            }
            PackagePath {
                return "/dotlrn/tlf-assignments"
            }
            PackageTitle {
                return "#tlf-assignments.mod_title#"
            }
            with_user_tracking {
                return 0
            }
            MenuBar:boolean {
                return 1
            }
            with_notifications {
                return 0
            }
            default {
                set res [next]
                return $res
            }
        }
    }

    Package instproc split_name {string} {
        return [list prefix "" suffix $string]
    }

    Package instproc initialize {} {
        ::xo::Page requireJS urn:ad:js:jquery
        ::xo::Page requireCSS "/resources/tlf-assignments/tlf-assignments.css"
        next
    }

    Package instproc require_folder_object {} {
        next
        [:folder_id] set_property -new 1 extra_menu_entries {
            {config -use assignments}
        }
    }
}

ad_proc -private ::tlf_cassignments::get_user_name {
    -user_id:required
} {
    retrieve complete name of user
} {
    switch [acs_object_type $user_id] {
        group {
            group::get -group_id $user_id -array ""
            return $(group_name)
        }
        user {
            return [xo::dc get_value  get_user "
            select  last_name || ' ' || first_names as name
            FROM cc_users
            where user_id = :user_id
        "  "unknown"]
        }
    }
}

ad_proc -private ::tlf_cassignments::after_instantiate {
    -package_id:required
} {
    Package after instantiation callback proc.
} {
    #nothing yet
}

ad_proc -private ::tlf_cassignments::get_all_counts {
    -package_id:required
    -display_members:required
    {-community_id ""}
} {
    Retrieve counts for all items in this package.

    @return a dict where keys are assignment_id + '_' +
    assignment_status and values are the counts grouped by assignment
    id and status
} {
    return [acs::per_request_cache eval -key \
                tlf_assignemtns_get_all_counts_${package_id}_${display_members}_${community_id} {
                    ::tlf_cassignments::get_all_counts_not_cached \
                        -package_id $package_id \
                        -display_members $display_members \
                        -community_id $community_id
    }]
}

ad_proc -private ::tlf_cassignments::get_all_counts_not_cached {
    -package_id:required
    -display_members:required
    {-community_id ""}
} {
    Retrieve counts for all items in this package.

    @return a dict where keys are assignment_id + '_' +
    assignment_status and values are the counts grouped by assignment
    id and status
} {
    set ms [clock milliseconds]

    set user_id [expr {[ns_conn isconnected] ? [ad_conn user_id] : 0}]

    if {$display_members} {
        if {$community_id eq ""} {
            if {[ns_conn isconnected]} {
                set community_id [dotlrn_community::get_community_id]
            } else {
                set community_id [::xo::dc get_value get_comm_id {
                    select min(community_id)
                      from dotlrn_community_applets
                     where package_id = :package_id
                }]
            }
        }
    } else {
        set community_id ""
    }

    set create_assignment_wf_id [string trimleft [$package_id resolve_page create-assignment.wf method_var] :]
    set abgabe_stud_wf_id       [string trimleft [$package_id resolve_page abgabe-stud.wf method_var] :]
    set parent_id [$package_id set folder_id]

    set counts [join [::xo::dc list_of_lists get_counts {
        select cast(bt.hkey -> 'assignment_id' as integer) || '_' ||
               case when bt.state = 'graded'
                  then 'graded'
                    when bt.state in ('submitted', 'grade_state', 'resubmission')
                  then 'submitted'
                    when bt.state in ('initial', 'rate')
                  then 'open'
               end,
               count(*)
          from xowiki_form_instance_item_index bt
         where page_template = :abgabe_stud_wf_id
           -- filter by group assignments in this package
        and cast(bt.hkey -> 'assignment_id' as integer)
        in (select item_id from xowiki_form_instance_item_index
            where page_template  = :create_assignment_wf_id
            and package_id     = :package_id
            and publish_status = 'ready'
            and parent_id      = :parent_id
            and (state = 'published' or acs_permission.permission_p(:package_id, :user_id, 'admin')))
        and bt.publish_status = 'ready'
        and bt.parent_id = :parent_id
        and ((bt.state = 'graded' and cast(bt.hkey->'graded' as boolean)) or
             bt.state <> 'graded')
        and not cast(bt.hkey->'amiadmin' as boolean)
        and (not :display_members or exists (select 1 from dotlrn_member_rels_approved
                                             where community_id = :community_id
                                             and user_id = cast(case when cast(bt.hkey->'foreign_user' as integer) > 0
                                                                then bt.hkey->'foreign_user'
                                                                else bt.hkey->'creator_id' end as integer)))
        group by 1
    }]]
    ns_log notice "tlf-assignments: getting counts from db took [expr {[clock milliseconds] - $ms}]"
    return $counts
}

ad_proc -private ::tlf_cassignments::get_from_value {
    -value:required
    -attribute:required
    {raw ""}
} {
    The value of a form entry might be:
    - an atomic list element
    - a list with attribute value pairs

    This function tries to obtain the queried attribute from the
    attribute value pair notation. If this fails, it returns a default
    value.

    taken from xowiki::formfield::file, needed if no initialized file
    formfield is available
} {
    set valueLength [llength $value]
    if {$valueLength > 1 && $valueLength %2 == 0} {
        array set "" $value
        if {[info exists ($attribute)]} {
            return $($attribute)
        }
    }
    return [join $raw]
}

Object ast_installer

ast_installer proc applet_key {} {
    return "tlf-assignments"
}

ast_installer proc my_package_key {} {
    return "tlf-assignments"
}

ast_installer ad_proc -private package_key {} {
    What package do I deal with?
} {
    return "tlf-assignments"
}

ast_installer ad_proc -private node_name {} {
    returns the node name
} {
    return "assignments"
}

ast_installer ad_proc -private pretty_name {} {
    returns the pretty name
} {
    return #tlf-assignments.mod_title#
}

ast_installer ad_proc -private add_applet {
} {
    One time init - must be repeatable!
} {
    ns_log notice "tlf_assignments:adding applet!"
    if {![dotlrn::is_package_mounted -package_key [:package_key]]} {
        set package_id [dotlrn::mount_package \
                            -url [:package_key] \
                            -package_key [:package_key] \
                            -pretty_name "#tlf-assignments.mod_title#" \
                            -directory_p t \
                           ]
        dotlrn_applet::add_applet_to_dotlrn \
            -applet_key [:applet_key] \
            -package_key [:my_package_key]
        ::xowf::Package initialize -package_id $package_id
        #import workflow and workflow items
        set wf_item [$package_id www-import-prototype-page workflow.wf]
        ns_log notice "wf_item $wf_item"
        #assignment creation workflow
        ::xowiki::FormPage create create_wf -noinit \
            -set object_type ::xowiki::FormPage \
            -set name create-assignment.wf \
            -set page_template [$wf_item set item_id]  \
            -title "Workflow Aufgabenerstellung" \
            -set text {} \
            -set nls_language {en_US} \
            -set package_id $package_id \
            -set parent_id [$package_id set folder_id] \
            -set anon_instances f \
            -set instance_attributes {
                workflow_definition {::xowf::include /packages/tlf-assignments/lib/create-assignment.wf}
            }
        create_wf save_new

        #submission workflow
        ::xowiki::FormPage create submit_wf -noinit \
            -set object_type ::xowiki::FormPage \
            -set name abgabe-stud.wf \
            -set page_template [$wf_item set item_id] \
            -title "Workflow Studentenabgabe" \
            -set text {} \
            -set package_id $package_id \
            -set parent_id [$package_id set folder_id] \
            -set nls_language {en_US} \
            -set anon_instances f \
            -set instance_attributes {
                workflow_definition {::xowf::include /packages/tlf-assignments/lib/assignment.wf}
            }
        submit_wf save_new

        #index page
        ::xowiki::Page create index_page \
            -set object_type ::xowiki::Page \
            -set name index \
            -set mime_type text/html \
            -set nls_language en_US \
            -set package_id $package_id \
            -set parent_id [$package_id set folder_id] \
            -set text {{{{tlf_assignment_list}}} text/html} \
            -set title #tlf-assignments.asts#
        index_page save_new

        ::xowiki::Page create groupmgr_page \
            -set object_type ::xowiki::Page \
            -set name groupmgr \
            -set mime_type text/html \
            -set nls_language en_US \
            -set package_id $package_id \
            -set parent_id [$package_id set folder_id] \
            -set text {{{{tlf_groupmgr}}} text/html} \
            -set title #tlf-assignments.groupmgr#
        groupmgr_page save_new
            
    }
}

ast_installer ad_proc -private  remove_applet {
} {
    One time destroy.
} {
    ad_return_complaint 1 "[:applet_key] remove_applet not implemented!"
}

ast_installer ad_proc -private add_applet_to_community {
    community_id
} {
    Add the xowiki applet to a specific dotlrn community
} {
    ns_log notice "tlf_assignments:adding applet to community"
    # get the community portal id
    set portal_id [dotlrn_community::get_portal_id -community_id $community_id]

    # get applet id
    set applet_id [dotlrn_applet::get_applet_id_from_key -applet_key [:applet_key]]
    # create the package instance
    set package_id [dotlrn::instantiate_and_mount \
                        -mount_point [:node_name]  \
                         $community_id [:package_key]]
    #add portlet
    set args [ns_set create]
    ns_set put $args package_id $package_id
    ns_set put $args param_action ""

    :add_portlet  $portal_id $args

    #::xowiki::Package initialize -package_id $package_id
    ns_log notice "tlf_assignments: adding applet to community pid $package_id"
    return $package_id
}

ast_installer ad_proc -private remove_applet_from_community {
    community_id
} {
    remove the applet from the community
} {
    # get package id
    set package_id [dotlrn_community::get_applet_package_id \
                        -community_id $community_id \
                        -applet_key [:applet_key]]
    # get portal id's
    set applet_id [dotlrn_applet::get_applet_id_from_key -applet_key [:applet_key]]
    #remove applet
    tlf_assignment_portlet::remove_self_from_page -portal_id $portal_id -package_id $package_id
    #unmount package
    dotlrn::unmount_package -package_id $package_id
}

ast_installer ad_proc -private add_user {
    user_id
} {
    one time user-specific init
} {
    # noop
}

ast_installer ad_proc -private remove_user {
    user_id
} {
    Remove user
} {
    # Just log the complaint instead of showing it to the user (script
    # would keep going anyway, as it is not aborted). Needed for
    # automated tests
    ns_log warning "[:applet_key] remove_user not implemented!"
    # ad_return_complaint 1 "[:applet_key] remove_user not implemented!"
    return
}

ast_installer ad_proc -private add_user_to_community {
    community_id
    user_id
} {
    Add a user to a specific dotlrn community
} {
    ns_log notice "adding user $user_id"
    set portal_id [dotlrn::get_portal_id -user_id $user_id]
    set package_id [dotlrn_community::get_applet_package_id -community_id $community_id -applet_key [:applet_key]]
    # use "append" here since we want to aggregate
    set args [ns_set create]
    ns_set put $args package_id $package_id
    ns_set put $args param_action append
    #ns_log notice "mmo adding user to community $package_id $community_id $user_id"
    :add_portlet $portal_id $args
    # flush cache for this portal
    #::tlf_global flush_portal $portal_id
    set this_user $user_id
    #package initialize
    if {[ns_conn isconnected]} {
        ::xowiki::Package initialize -package_id $package_id
        #ns_log notice "did a normal package init on connected"
    } else {
        ::xowiki::Package initialize -package_id $package_id -init_url false \
            -url [site_node::get_url_from_object_id -object_id $package_id] -user_id [acs_magic_object "sys_account"] -actual_query ""
        #ns_log notice "did a package init on not connected"
    }
    #activate deactivated hw
    set sub_template [string trimleft [$package_id resolve_page abgabe-stud.wf method_var] :]
    set user_submissions [xo::dc list sub_list "select item_id from
    xowiki_form_instance_item_index
    where package_id = :package_id
    and page_template = :sub_template
    and (hkey->'group_id')::text = :this_user"]
    if {[llength $user_submissions] > 0} {
        ns_log notice $user_submissions
        foreach subm $user_submissions {
            ::xo::xotcl_object_cache flush $subm
            set revision_id [xo::dc get_value db_rev "select latest_revision
                from cr_items where item_id = :subm"]
            ns_log notice "revision_id $revision_id"
            ::xo::db::sql::content_item set_live_revision \
                -revision_id $revision_id \
                -publish_status "ready"
            ns_log notice "enabled  $subm of user $this_user"
        }
    }
    #adjust groups
    #::tlf_groupmgr::add_user_to_community $community_id $this_user
}

ast_installer ad_proc -private remove_user_from_community {
    community_id
    user_id
} {
    Remove a user from a community
} {
    set package_id [dotlrn_community::get_applet_package_id -community_id $community_id -applet_key [:applet_key]]
    set portal_id [dotlrn::get_portal_id -user_id $user_id]
    set args [ns_set create]
    ns_set put $args package_id $package_id
    # ns_log notice "mmo removing portlet $package_id $community_id $user_id"        :remove_portlet $portal_id $args
    # flush cache for this portal
    #::tlf_global flush_portal $portal_id
    #set user submissions inactive
    set result [list]
    set level [info level]
    incr level -1
    for { set i 1 } { $i <= $level } { incr i 1 } {
        lappend result [lindex [info level $i] 0]
    }
    foreach res $result {
        if {[regexp {^.*(dotlrn_community::archive)$} $res match]} {
            ns_log notice "$community_id - remove_user_from_community called from dotlrn_community::archive, terminate!"
            return
        }
    }
    #if this is a shared course and the user is registered in another community, do not archive the submissions!
    set key [ast_installer applet_key]
    set user_active_in_shared_pkg [xo::dc list shared_pkg "
        select community_id
          from dotlrn_member_rels_approved
         where community_id in (
               select community_id from dotlrn_community_applets
                where package_id = :package_id)
           and rel_type in ('dotlrn_student_rel','dotlrn_member_rel')
           and user_id = :user_id
           and community_id != :community_id"]
    if {$user_active_in_shared_pkg ne ""} {
        ns_log notice "user $user_id is still active in shared package $user_active_in_shared_pkg, do not deactivate submissions!"
        return
    }
    set this_user $user_id
    if {[ns_conn isconnected]} {
        ::xowiki::Package initialize -package_id $package_id
        # ns_log notice "mmo did a normal package init on connected"
    } else {
        ::xowiki::Package initialize -package_id $package_id \
            -init_url false \
            -url [site_node::get_url_from_object_id -object_id $package_id] \
            -user_id [acs_magic_object "sys_account"] \
            -actual_query ""
        # ns_log notice "mmo did a package init on not connected"
    }

    set submission_template [string trimleft [$package_id resolve_page abgabe-stud.wf method_var] :]
    set user_submissions [xo::dc list sub_list "
        select item_id
        from xowiki_form_instance_item_index
        where package_id = :package_id
        and (hkey->'group_id')::text = :this_user
        and page_template = :submission_template"]
    if {[llength $user_submissions] > 0} {
        # ns_log notice $user_submissions
        foreach subm $user_submissions {
            ::xo::xotcl_object_cache flush $subm
            set revision_id [xo::dc get_value db_rev "
                select latest_revision
                from cr_items
                 where item_id = :subm"]
            # ns_log notice "revision_id $revision_id"
            ::xo::db::sql::content_item set_live_revision \
                -revision_id $revision_id \
                -publish_status "production"
            # ns_log notice "removed $subm of user $user_id"
        }
    }
    #call groupmgr to remove user from groups
    # ns_log notice "calling groupmr"
    ::tlf_groupmgr::remove_user_from_community $community_id $user_id
    # ns_log notice "done"    
}

ast_installer ad_proc -private add_portlet {
    portal_id
    args
} {
    A helper proc to add the underlying portlet to the given portal.

    @param portal_id
} {
    tlf_assignment_portlet::add_self_to_page \
        -portal_id $portal_id \
        -package_id [ns_set get $args "package_id"] \
        -param_action [ns_set get $args "param_action"]
    #flush cached portal objects
#    ns_cache flush tlf_global_cache portal.${portal_id}
}

ast_installer ad_proc -private remove_portlet {
    portal_id
    args
} {
    A helper proc to remove the underlying portlet from the given portal.

    @param portal_id
    @param args A list of key-value pairs (possibly user_id, community_id, and more)
} {
    tlf_assignment_portlet::remove_self_from_page \
        -portal_id $portal_id \
        -package_id  [ns_set get $args "package_id"]
    #flush cached portal objects
#    ns_cache flush tlf_global_cache portal.${portal_id}
}

ast_installer ad_proc -private clone {
    old_community_id
    new_community_id
} {
    Clone this applet's content from the old community to the new one
} {
    #not implemented
}

ast_installer ad_proc -private change_event_handler {
    community_id
    event
    old_value
    new_value
} {
    listens for the following events:
} {
    #; nothing
}

ast_installer proc install {} {
    ns_log notice "xolrn:doing an install of tlf_assignments package"
    set name [:applet_key]
    ::xo::dc transaction {
        # register the applet implementation
        ::xo::db::sql::acs_sc_impl new \
            -impl_contract_name "dotlrn_applet" -impl_name $name \
            -impl_pretty_name "" -impl_owner_name $name
        # add the operations

        foreach {operation call} {
            GetPrettyName         "ast_installer pretty_name"
            AddApplet                 "ast_installer add_applet"
            RemoveApplet              "ast_installer remove_applet"
            AddAppletToCommunity      "ast_installer add_applet_to_community"
            RemoveAppletFromCommunity "ast_installer remove_applet_from_community"
            AddUser                   "ast_installer add_user"
            RemoveUser                "ast_installer remove_user"
            AddUserToCommunity        "ast_installer add_user_to_community"
            RemoveUserFromCommunity   "ast_installer remove_user_from_community"
            AddPortlet                "ast_installer add_portlet"
            RemovePortlet             "ast_installer remove_portlet"
            Clone                     "ast_installer clone"
            ChangeEventHandler        "ast_installer change_event_handler"
        } {
            ::xo::db::sql::acs_sc_impl_alias new \
                -impl_contract_name "dotlrn_applet" -impl_name $name  \
                -impl_operation_name $operation -impl_alias $call \
                -impl_pl "TCL"
        }

        # Add the binding
        ::xo::db::sql::acs_sc_binding new \
            -contract_name "dotlrn_applet" -impl_name $name
        :add_applet
    }
}

ast_installer proc uninstall {} {
    :log "--doing an uninstall of tlf-assignments"
    #
    # pretty similar "xowiki_portlet uninstall"
    #
    set name [:applet_key]

    ::xo::dc transaction {
        #
        # get the datasource
        #

        set ds_id [xo::dc get_value  dbqd..get_ds_id {
            select datasource_id from portal_datasources where name = :name
        }  "0"]

        if {$ds_id == 0} {
            ns_log notice "No datasource id found for $name"
        } else {
            #
            # drop the datasource
            #
            ::xo::db::sql::portal_datasource delete -datasource_id $ds_id
        }
        #
        #  drop the operation
        #
        foreach operation {
            GetPrettyName
            AddApplet
            RemoveApplet
            AddAppletToCommunity
            RemoveAppletFromCommunity
            AddUser
            RemoveUser
            AddUserToCommunity
            RemoveUserFromCommunity
            AddPortlet
            RemovePortlet
            Clone
        } {
            ::xo::db::sql::acs_sc_impl_alias delete \
                -impl_contract_name "dotlrn_applet" -impl_name $name \
                -impl_operation_name $operation
        }
        #
        #  drop the binding
        #
        ::xo::db::sql::acs_sc_binding delete \
            -contract_name "dotlrn_applet" -impl_name $name

        #
        #  drop the implementation
        #
        ::xo::db::sql::acs_sc_impl delete \
            -impl_contract_name "dotlrn_applet" -impl_name $name

        #remove dotlrn_applet
        ::xo::dc dml delete_applet {
            delete
            from dotlrn_applets
            where applet_key = :name
        }
    }
    :log "finished uninstall of tlf-assignments"        
}

namespace eval ::xowf {

    AssignmentCoreWFprocs instproc build_name {{-nls_language ""}} {
        set name ${:name}
        return $name
    }

    AssignmentCoreWFprocs instproc is_link_page {} {
        return 0
    }

    AssignmentCoreWFprocs instproc footer {} {
        return ""
    }
}

::xo::library source_dependent

#
# Local variables:
#    mode: tcl
#    tcl-indent-level: 4
#    indent-tabs-mode: nil
# End:
