##::xo::db::require package xowf
::xo::library require -package xowf xowf-procs


namespace eval ::xowf {

    Class create ::xowf::AssignmentState
    ##########################################
    # State initial
    ##########################################
    AssignmentState instproc create_assign_initial_loader {name} {
        #create a new assignment, or edit an existing one (with fewer options)
        ::xo::Page requireJS "/resources/tlf-assignments/tlf-assignment.js"
        ::xo::Page requireCSS "/resources/tlf-assignments/tlf-assignments.css"
        set obj ${:object}
        set memberhtml ""
        set member_names ""
        set group_names ""
        set package_id [${:object} set package_id]
        set bearb [${:object} get_property -name bearbeiter -default ""]
        foreach gr [${:object} get_property -name groups -default ""] {
            append group_names "<a href='groupmgr?a=view_detail&id=$gr'>\
            [::xo::dc get_value group_name {select group_name from groups where group_id = :gr}]</a>
            <br>"
        }
        foreach member $bearb {
            append member_names [::tlf_cassignments::get_user_name -user_id $member]<br>
        }
        if {$group_names ne ""} {
            set memberhtml "<div class='form-item-wrapper'>
                                <div class='form-label'>[_ tlf-assignments.assigned_members]:</div>
                                <div class='form-widget'>$group_names</div>
                            </div>"
        } elseif {$member_names ne ""} {
            set memberhtml "<div class='form-item-wrapper'>
                                <div class='form-label'>[_ tlf-assignments.assigned_members]:</div>
                                <div class='form-widget'>$member_names</div>
                            </div>"
        }
        set exam_vis [expr {[${:object} get_property -name exercise_form] eq "exam" ? "" : "display:none"}]
        set ot_vis [expr {[${:object} get_property -name exercise_form] eq "ot" ? "" : "display:none"}]
        #todo: avoid querying db two times about calc_submissions
        array set subm [$obj calc_submissions "submitted"]
        set is_not_graded [expr {$subm(graded) eq 0}]
        set disableme [expr {!$is_not_graded && [$obj set state] ne "initial" ? "disabled" : ""}]
        
        #generate formdef depending on state
        set formdef "<form>"
        if {[$obj set state] eq "initial"} {
            append formdef "
            <div class='table-wizard'>
                <div class='tr'>
                <div class='td active'>[_ tlf-assignments.create_1]</div>
                <div class='td grey'>[_ tlf-assignments.create_2]</div>
                </div>
            </div>"
        }
        append formdef "
            @_title@
            @text@
            @ss1@
            [${:object} multiple_files ss]
            @_description@
        "
        if {[$obj set state] eq "initial"} {
            append formdef "
                <hr>
                @exercise_form@
                <div style='$ot_vis' id='ot_conditional'>@opentext_sample@</div>
            "
        } elseif {[$obj get_property -name exercise_form] eq "exam"} {
            append formdef "
            @examvis@
            $updatelink
            "
        }
        set release_date_vis [expr {[$obj get_property -name beurteilung] eq "date" ? "" : "display:none;"}]
        append formdef "
             <hr>
              @time_from@
              @time_to@
              @verbergen@
            <hr>
            @beurteilung@<div style='$release_date_vis' id='date_conditional'>@release_date@</div>
        "
        if {$disableme eq ""} {
            append formdef "
                <p>@modus@</p>
                <div style='[expr {[$obj get_property -name points] eq {} ? {display:none;} : {}}]' id='grade_conditional'>@points@</div>
            "
            }
        append formdef "
            $memberhtml
        "
        
        if {[$obj set state] eq "initial"} {
	    set nonce [security::csp::nonce]
            append formdef "
                <script nonce='$nonce'>
                    var beurt = \$('F.[${:object} set name].beurteilung').value;
                    if (beurt == 'date') {\$('date_conditional').style.display = '';}
                    var modus = \$('F.[${:object} set name].modus').value;
                    if (modus == 'points') {\$('grade_conditional').style.display = '';}
                    var eform = \$('F.[${:object} set name].exercise_form').value;
                    if (eform == 'ot') {\$('ot_conditional').style.display = '';}
                    if (eform == 'exam') {\$('exam_conditional').style.display = '';}
                    //do not allow empty points in assignments to be graded with points
                    \$(document).ready(function() {
                        \$('.form-button button').click(function(e) {
                            if ( \$(\"select\[name='modus'\]\").val() == 'points' ) {
                                if (\$(\"input\[name='points'\]\").val() == '') {
                                    alert('#tlf-assignments.maxpoints_missing#');
                                    return false;
                                }
                            }
                            return true;
                        });
                    });
                </script>
            "
        }
        append formdef "</form> "

        # generate options for the submission types
        set submission_type_options [list]
        foreach sub_type [list ::SubmissionType=file ::SubmissionType=ot ::SubmissionType=textarea] {
             set option [$sub_type asOption]
             if {$option ne ""} {
                lappend submission_type_options [$sub_type asOption]
             }
        }

        #generate options for grading types
        set grading_type_options [list]
        foreach sub_type [GenericGradingType info subclass] {
            set option [$sub_type asOption]
            if {$option ne ""} {
                lappend grading_type_options [$sub_type asOption]
            }
        }
        
        set statusform [::xowiki::Form new -childof [self] -name en:initial_loader \
            -form $formdef \
            -anon_instances t \
            -form_constraints [subst {
                {_title:text,label=#tlf-assignments.title#,required=yes}
                {_description:label=#tlf-assignments.add_inf#,\
                    help_text=#tlf-assignments.desc_help#}
                {text:richtext,\
                    editor=ckeditor4,\
		    label=#tlf-assignments.text#,\
		    customConfig=/resources/tlf-assignments/ck_config_minimal.js,
                    toolbar=Minimal}
                ss1:pretty_file,label=#tlf-assignments.attachment#
                ss2:pretty_file,label=
                ss3:pretty_file,label=
                ss4:pretty_file,label=
                ss5:pretty_file,label=
                state:label,label=Status,hidden
                {points:numeric,\
                    format=%.2f,\
                    size=3,\
                    label=max. #tlf-assignments.points#,\
                    validator=numeric,\
                    locale=[::xo::cc set locale]}
                {_page_order:hidden}
                _nls_language:hidden
                _creator:hidden
                release_date:date_with_widget,format=DD_MONTH_YYYY_HH24_MI,label=#tlf-assignments.rel_date#
                _name:hidden
                {exercise_form:conditional_ff,label=#tlf-assignments.type#,
                    options=$submission_type_options,\
                        onselection=ot.exam,\
                        condition=ot_conditional.exam_conditional,\
                        default=file,
                        required=yes}
                {examvis:select,
                    options={#tlf-assignments.instant_share# sofort}\
                    {{#tlf-assignments.time_share#} timeup}\
                    {#tlf-assignments.no_share# never},
                    default=timeup,
                    required=yes,
                    label=#tlf-assignments.sample_sol#}
                {beurteilung:conditional_ff,label=#tlf-assignments.grade_release#,\
                    options={{#tlf-assignments.rel_sofort#} sofort} \
                            {#tlf-assignments.date# date} \
                            {{#tlf-assignments.individual#} individual} \
                            {{#tlf-assignments.none#} keine},\
                        onselection=date,\
                        condition=date_conditional,\
                        default=sofort, \
                        required=yes}
                {modus:conditional_ff,label=#tlf-assignments.grade_mode#,\
                    options=$grading_type_options,\
                    onselection=points,\
                    condition=grade_conditional,\
                    default=note, \
                    required=yes}
                {verbergen:checkbox,label=#tlf-assignments.verbergen#,\
                    options={{#tlf-assignments.nein#} 1},\
                    default=0}
                "time_from:date_with_widget,format=DD_MONTH_YYYY_HH24_MI,\
                    label=#tlf-assignments.available-from#"
                "time_to:date_with_widget,format=DD_MONTH_YYYY_HH24_MI,\
                    validator=end_time,\
                    label=#tlf-assignments.available-until#"
                {opentext_sample:opentext_compound,\
                    label=Antworten,\
                    validator=valid,\
                    help_text=#tlf-assignments.help_ot#}
        }]]
        return $statusform
    }

    ##########################################
    # State editing
    ##########################################    
    AssignmentState instproc create_assign_published {name} {
        #assignment has been published
        #allow editing without changing the state of the item
        set package_id [::xo::cc set package_id]
        
        #don't allow students to see instance attributes by guessing m edit
        if {![permission::permission_p -no_login \
             -party_id [::xo::cc user_id] \
             -object_id [${:object} item_id] \
             -privilege admin]} {
                set statusform [::xowiki::Form new -childof [self] -name en:initial_loader \
                -form "
                    <form> 
                    [${:object} render_angabe]
                    </form> 
                " \
                -anon_instances t \
                -form_constraints [subst {
                {_title:hidden}
                {_description:hidden}
                _page_order:hidden _nls_language:hidden
                _creator:hidden
                _name:hidden
                }] \
                ]
            return $statusform
            }
        
        if {[::xo::cc query_parameter m] eq "edit"} {
            return [:create_assign_initial_loader $name]
        }
        set html "[${:object} render_angabe_full]"
        append html "<a href='[${:object} pretty_link]?m=submit' class='button'>[_ tlf-assignments.stud_view]</a>"
        #::xo::Page set_property doc menubar [$package_id MenuBar -context_item ${:object}]

        set statusform [::xowiki::Form new -childof [self] -name en:initial_loader \
            -form "
                <form> 
                    $html
                </form> 
            " \
            -anon_instances t \
            -form_constraints [subst {
                {_title:hidden}
                {_description:hidden}
                _page_order:hidden _nls_language:hidden
                _creator:hidden
                _name:hidden
            }] \
        ]
        return $statusform 
    }    

    ##########################################
    # State assign
    ##########################################
    AssignmentState instproc create_assign_studentmgr {name} {
            #form_loader for assigning students/group to an exercise
            #this is step 2 in exercise creation
            #user can either select all community members, single members or groups
            ::xo::Page requireJS "/resources/tlf-assignments/tlf-assignment.js"
            ::xo::Page requireCSS "/resources/tlf-assignments/tlf-assignments.css"
            set ef [${:object} get_property -name exercise_form]
            #retrieve all groups
            set groups ""
            set package_id [${:object} set package_id]
            set applet_key [ast_installer applet_key]
            ::xo::dc foreach comm "select community_id as comm_id from dotlrn_community_applets
                where package_id = :package_id and applet_id = (select applet_id from dotlrn_applets
                where applet_key = :applet_key)" {
                set gname groupmgr_${comm_id}
                ::xo::dc foreach gr "select group_id, group_name from groups where description = :gname" {
                    regsub -all {,} $group_name __COMMA__ group_name
                    lappend groups [subst {"$group_name" "$group_id"}]
                }
            }       
        
            #who can edit this exercise?
            set options ""
            foreach elegible_type [SubmissionType=$ef elegible_types] {
                append options [subst {{{#tlf-assignments.${elegible_type}#} {$elegible_type}}\n}]
            }

            #only one option will be visible, others will be hidden
            set member_vis [expr {[${:object} get_property -name zuweisen] eq "einzelne" ? "" : "display:none"}]
            set group_vis [expr {[${:object} get_property -name zuweisen] eq "gruppe" ? "" : "display:none"}]
            set group_ff "candidate_box_select_jquery,
                        options=$groups,
                        label=[_ tlf-assignments.gruppe],
                        help_text=[_ tlf-assignments.ziehen_help1]"
            set member_ff "candidate_box_select_jquery,
                        role=tlf_ast_group_stud,
                        label=[_ tlf-assignments.einzelne],
                        help_text=[_ tlf-assignments.ziehen_help2]"
            if {![info exists comm_id]} {
                #during automated tests we might not have a community_id
                set teilnehmer {}
            } else {
                set teilnehmer [::xo::role::tlf_ast_group_stud get_members -object_id $comm_id]
            }
            #if no groups or members exist, show a placeholder text
            set placeholder_no_groups "[_ tlf-assignments.empty_groups]"
            set placeholder_no_members "[_ tlf-assignments.empty_com]"
            set member_help ""
            set help_group ""
            if {$groups eq ""} {
                set group_ff hidden
                set help_group $placeholder_no_groups
            }
            if {$teilnehmer eq ""} {
                set member_ff hidden
                set member_help $placeholder_no_members
            }
	    set nonce [security::csp::nonce]
            set statusform [::xowiki::Form new -childof [self] -name en:initial_loader \
                -form "
                    <form>
                        <div class='table-wizard'>
                            <div class='tr'>
                                <div class='td grey'>
                                   [_ tlf-assignments.create_1]
                                </div>
                                <div class='td active'>
                                    [_ tlf-assignments.create_2]
                                </div>
                            </div>
                        </div>
                        @zuweisen@
                        <div id='member_conditional' style='$member_vis'>@bearbeiter@<p>$member_help</p></div>
                        <div id='group_conditional' style='$group_vis'>@groups@ <p>$help_group</p></div>
                        <script nonce='$nonce'>
                            //if user reloads the page, make sure the correct conditional is visible
                            var zuweisen = \$('F.[${:object} set name].zuweisen').value;
                            if (zuweisen == 'gruppe') {\$('group_conditional').style.display = '';}
                            if (zuweisen == 'einzelne') {\$('member_conditional').style.display = '';}
                        </script>
                    </form>
                " \
                -anon_instances t \
                -form_constraints [subst {
                    {zuweisen:conditional_ff,required=1,label=#tlf-assignments.zuweisen_help#,
                        options=$options,
                        default=alle,
                        onselection=einzelne.gruppe,
                        condition=member_conditional.group_conditional,
                        validator=members_valid
                    }
                    {bearbeiter:$member_ff}
                    {groups:$group_ff}
                    onselection=ot.exam,
                    condition=ot_conditional.exam_conditional,
                    default=file
                    @cr_fields:hidden
                }] ;#end subst
          ] ;#end statusform
          return $statusform
    }    
    

}
::xo::library source_dependent
